=begin
Copyright 2010 (c) TIG
  All Rights Reserved.
  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED 
  WARRANTIES,INCLUDING,WITHOUT LIMITATION,THE IMPLIED WARRANTIES OF 
  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
###
  CymaCurveTool.rb
###
Usage:
  Choose 'Cyma Curve Tool' from the Plugins Menu;
  or type 'cyma' in the Ruby Console...
###
  Select parameters in the dialog, on OK the specified Cyma Curve 
  appears on the end of your cursor - place it where you wish...
  The parameters are:
  Overall Size X: The Horizontal size of the enclosing square - 
                default 100mm or 4" depending on model's units, 
                the last entered value is remembered with the model.
  Overall Size Y: The Vertical size of the enclosing square - 
                default 100mm or 4" depending on model's units, 
                the last entered value is remembered with the model.
  Chord Proportion:
                The proportion of the Curve's Arc's Chord to its Radius.
                There are two possibilities...
                1: assuming a 'square form' [X==Y]...
                For a basic Cyma it's sqrt(2) [1.4142136]
                  giving tangential springing points [90 degree].
                For a "Palladian" Cyma it's 7/6 [1.1666666]
                  giving flattened arcs to the curve [the default].
                The smaller the value the flatter the arcs, so 1 gives
                  equilateral arcs which can look too flat when viewed 
                  from below; values <0.1 will given arcs so flat you 
                  could have used straight lines! so 0.1 is the minimum.
                You can enter values in decimal format [1.2345], 
                  fractional format [7/6] or as 'sqrt' [sqrt(2)]
                  [note format with (nn) etc]
                2: assuming a 'rectangular' form [X!=Y] the 90 degree 
                  'tangential' form can be set by giving the 
                  Chord to Radius proportion based on the formula
                    2*Y/sqrt(X*X+Y*Y)
                  so you can calculate this beforehand and enter it as 
                  a decimal value, e.g. it's 0.8944 when X=200 & Y=100, 
                  OR as it's also shown in the VCB prompt once the X & Y 
                  values are set: to get it calculated for you, you make 
                  a rectangular Cyma using the default value and then 
                  immediately undo it, then rerun the tool - all of the 
                  dialog's values are remembered, but the VCB prompt now 
                  also shows the value that would give a tangential 
                  90 degree form - so you can type that value into the 
                  dialog and get 90 degree tangential springing points 
                  for rectangular forms too.
                  Otherwise you can use the 7/6 or other values as 
                  desired...
  Curve Type:   The two types of curve available are:
                'Cyma Recta' - also called 'Gola'
                 [starts convex, ends concave] - it is the default type.
                and
                'Cyma Reversa' - also called 'Ogee'
                 [starts concave, ends convex]
  Curve Format: You choose from the following:
                'As Curve' - just the two arcs welded as a curve.
                'As Curve with Face' - the two arcs welded as a curve 
                          and their internal edges with a face - 
                          useful for extrusions.
                'As Curve with Guides' - the two arcs welded as a curve 
                          with guide-lines and -points added to show the 
                          arcs' centers etc.
  Segments:     Default is 36 - enter new value, must be 3 or more - 
                more segments give smoother curves BUT more geometry !
  The geometry is in a Component named 'CymaCurve#1' or "CymaCurve#2" 
  etc for multiple versions...
  Its description reflects its settings...
  It is attached to your cursor by its bottom left corner 
  springing-point and it snaps to faces.  
  You can mirror it by using Scale -1 or Move/Copy it as desired, 
  or Explode as required...
###
Donations:
  Are welcome [by PayPal], please use 'TIGdonations.htm' in the 
  ../Plugins/TIGtools/ folder.
###
Version:
  1.0 20100307 First Release.
  1.1 20100308 FR lingvo added by Pilou. Faced option curves now welded.
  1.2 20100308 Buggy second arc formation fixed.
  1.3 20100308 Missing require deB... added.
  1.4 20100311 Non-square forms now possible and sizes entered, X & Y.
               VCB prompt now shows the chord-proportion value needed to 
               get tangential ends - useful if it's a non-square form.
               The arcs' segments can now be set.
               'Gola' and 'Ogee' added to type descriptions.
               Notes and lingvo files updated to match changes.
  1.5 20100311 Broken curve glitch fixed.
###
=end
#-----------------------------------------------------------------------
require('sketchup.rb')
load('deBabelizer.rb')
###

class CymaCurveTool


def db(string)
  dir=File.dirname(__FILE__)+"/TIGtools"
  toolname="CymaCurveTool" 
  locale=Sketchup.get_locale.upcase
  path=dir+"/"+toolname+locale+".lingvo"
  if not File.exist?(path)
    return string
  else
    deBabelizer(string,path)
  end 
end#db
###---------------------------------------


def activate
   @ip=Sketchup::InputPoint.new
   @ip1=Sketchup::InputPoint.new
   @model=Sketchup.active_model
   return nil if not self.dialog
   ### calculate dims
   sizeX=@sizeX
   sizeY=@sizeY
   chordText=@chord
   type=@type
   format=@format
   segs=@segs
   ### get chord prop from TXT
   if chordText=~/\//
     num=chordText.split('/')[0].to_f
     den=chordText.split('/')[1].to_f
     chord=num/den
   elsif chordText=~/^sqrt\(/
     chord=Math::sqrt(chordText.gsub(/^sqrt\(/,'').to_f)
   else ### decimal
     chord=chordText.to_f
   end#if
   ### error traps
   if chord<0.1
     UI.messagebox(db("Chord Proportion cannot be < 0.1: reset to 0.1."))
     @chord="0.1"
     @model.set_attribute("CymaCurve","chord",@chord)
     chord=0.1
   end#if
   if @segs<3
     UI.messagebox(db("An Arc's Segments must be >= 3: reset to 3."))
     @segs=3
     @model.set_attribute("CymaCurve","segs",@segs)
     segs=3
   end#if
   ### 
   ###
   p0=[0,0,0]
   p1=[sizeX,sizeY,0]
   p01=[sizeX/2,sizeY/2,0]
   p0c=[sizeX/4,sizeY/4,0]
   p1c=[3*sizeX/4,3*sizeY/4,0]
   apex=[0,sizeY,0]
   if type==(db("Cyma Recta [Gola]"))
     recta=true
     ang= 90.degrees
   else
     recta=false
     ang= -90.degrees
   end#if
   chordLength=p0.distance(p01)
   opp=chordLength/2
   radius=chordLength/chord
   adj=Math::sqrt((radius*radius)-(opp*opp))
   vec=p0.vector_to(p1)
   tr=Geom::Transformation.rotation(p0c,[0,0,1],ang)
   vec.transform!(tr)
   ct0=p0c.clone.offset!(vec,adj)
   ###
   vec=p0.vector_to(p1)
   tr=Geom::Transformation.rotation(p0c,[0,0,1],-ang)
   vec.transform!(tr)
   ct1=p1c.clone.offset!(vec,adj)
   ###
   if Sketchup.version[0,1].to_i > 6
	 @model.start_operation((db("Cyma Curve")),true)
     ### 'false' is best to see results as UI/msgboxes...
   else
	 @model.start_operation((db("Cyma Curve")))
   end#if
   ###
   @grp=@model.active_entities.add_group()
   ents=@grp.entities
   ###
   ang1=(ct0.vector_to(p01)).angle_between(X_AXIS)
   ang2=(ct0.vector_to(p0)).angle_between(X_AXIS)
   if @sizeX<@sizeY
     if recta
       ang1= -ang1
       ang2= ang2
     else
       ang1= ang1
       ang2= 360.degrees-ang2
     end#if
   end#if
   ###
   if recta
     arc1=ents.add_arc(ct0,[1,0,0],[0,0,1],radius,-ang1,-ang2,segs)
   else
     arc1=ents.add_arc(ct0,[1,0,0],[0,0,1],radius,ang1,ang2,segs)
   end#if
   ###
   ###
   ang1=(ct1.vector_to(p01)).angle_between(X_AXIS)
   ang2=(ct1.vector_to(p1)).angle_between(X_AXIS)
   if @sizeX<@sizeY
     if recta
       ang1= -ang1
       ang2= -360.degrees+ang2
     else
       ang1= ang1
       ang2= -ang2
     end#if
   end#if
   ###
   if recta
     arc2=ents.add_arc(ct1,[1,0,0],[0,0,1],radius,ang2,ang1,segs)
   else
     arc2=ents.add_arc(ct1,[1,0,0],[0,0,1],radius,-ang2,-ang1,segs)
   end#if
   ###
   ### weld edges
     edges=arc1.reverse+arc2.reverse
     verts=[]
     edges.each{|e|verts<<e.vertices[0]<<e.vertices[1]}
     verts.uniq!
     pts=[]
     verts.each{|v|pts<<v.position.to_a}
     pts.uniq!
     edges.each{|e|e.erase! if e.valid?}
     ents.add_curve(pts)
   ###
   ### do other formats
   if format==(db("As Curve with Face"))
     ents.add_line(p0,apex)
     ents.add_line(apex,p1)
     edges=[]
     ents.each{|e|edges<<e}
     edges.each{|e|break if e.find_faces>0}
   elsif format==(db("As Curve with Guides"))
     ents.add_cpoint(apex)
     ents.add_cpoint(ct0)
     ents.add_cpoint(ct1)
     ents.add_cline(p0,apex)
     ents.add_cline(p1,apex)
     ents.add_cline(ct0,p0)
     ents.add_cline(ct0,p01)
     ents.add_cline(ct1,p1)
     ents.add_cline(ct1,p01)
     ents.add_cline(p0,p01)
     ents.add_cline(p1,p01)
     ents.add_cline(ct0,p0c)
     ents.add_cline(ct1,p1c)
   end#if
   ### make compo
   inst=@grp.to_component
   ### make it sticky
   gdef=inst.definition
   gdef.description=@sizeX.to_s+":"+@sizeY.to_s+":"+@chord+":"+@type+":"+@format
   gdef.name=(db("CymaCurve"))+"#1"
   ###
   inst.erase! if inst.valid?
   ###
   gdef.behavior.is2d=true
   gdef.behavior.snapto=SnapTo_Arbitrary
   ### attach to cursor
   @model.place_component(gdef,false)###true=repeat
   ###
   @model.commit_operation
   return nil
   ###
end#def


def deactivate(view)
  ### view.invalidate if view ###
  Sketchup::set_status_text("")
  Sketchup.send_action "selectSelectionTool:"
  return nil
end # deactivate
    
    
def resume(view)
  Sketchup::set_status_text(@msg)
  ### view.invalidate if view ###
end #resume


def dialog()
  sizeX=4.inch if @model.options["UnitsOptions"]["LengthUnit"]<2
  sizeX=100.mm if @model.options["UnitsOptions"]["LengthUnit"]>1
  sizeY=4.inch if @model.options["UnitsOptions"]["LengthUnit"]<2
  sizeY=100.mm if @model.options["UnitsOptions"]["LengthUnit"]>1
  types=["Cyma Recta [Gola]","Cyma Reversa [Ogee]"]
  format0=(db("As Curve"))
  format1=(db("As Curve with Face"))
  format2=(db("As Curve with Guides"))
  formats=[format0,format1,format2]
  ###
  @sizeX=@model.get_attribute("CymaCurve","size",sizeX)
  @sizeY=@model.get_attribute("CymaCurve","sizeY",sizeY)
  @chord=@model.get_attribute("CymaCurve","chord","7/6")
  @type=@model.get_attribute("CymaCurve","type",types[0])
  @format=@model.get_attribute("CymaCurve","format",formats[0])
  @segs=@model.get_attribute("CymaCurve","segs",36)
  ###
  types=types.join('|')
  formats=formats.join('|')
  ###
  prompt0=(db("Overall Size X: "))
  prompt0a=(db("Overall Size Y: "))
  prompt1=(db("Chord Proportion: "))
  prompt2=(db("Curve Type: "))
  prompt3=(db("Curve Format: "))
  prompt4=(db("Arcs' Segments: "))
  prompts=[prompt0,prompt0a,prompt1,prompt2,prompt3,prompt4]
  title=(db("Cyma Curve Parameters"))
  values=[@sizeX,@sizeY,@chord,@type,@format,@segs]
  popups=["","","",types,formats,""]
  ###
  if @sizeX==@sizeY
    proptxt="sqrt(2) [1.4142136]"
  else
    prop=2*@sizeY/(Math::sqrt((@sizeX*@sizeX)+(@sizeY*@sizeY)))
    proptxt=prop.to_s
  end#if
  @msg=(db("Chord Proportion: Palladian = 7/6, Equilateral = 1, Tangential = "))+proptxt
  Sketchup::set_status_text(@msg)
  ###
  results=inputbox(prompts,values,popups,title)
  return nil if not results
### do processing of results...
@sizeX=results[0]
@sizeY=results[1]
@chord=results[2]
@type=results[3]
@format=results[4]
@segs=results[5]
###
@model.set_attribute("CymaCurve","size",@sizeX)
@model.set_attribute("CymaCurve","sizeY",@sizeY)
@model.set_attribute("CymaCurve","chord",@chord)
@model.set_attribute("CymaCurve","type",@type)
@model.set_attribute("CymaCurve","format",@format)
@model.set_attribute("CymaCurve","segs",@segs)
###
return true
###
end #def dialog

###################################################---------------------
end#class


def cyma()
  Sketchup.active_model.select_tool(CymaCurveTool.new)
  puts
end#def

#--------- menu -----------------------------
if not file_loaded?(__FILE__)
   txt0="Cyma Curve Tool"
   dir=File.dirname(__FILE__)+"/TIGtools"
   toolname="CymaCurveTool"
   locale=Sketchup.get_locale.upcase
   path=dir+"/"+toolname+locale+".lingvo"
   if File.exist?(path)
     txt0=deBabelizer(txt0,path)
   end#if
   UI.menu("Plugins").add_item(txt0){cyma()}
end#if
file_loaded(__FILE__)
#---------------------------------
